@@ -1,17 +1,20 @@
 # -*- coding: utf-8 -*-
 
 """
-Module implementing Things API.
-"""
+Module implementing the Things API.
 
-__version__ = "0.0.7"
+The terms of the API aim to match those used in the Things app and
+Things URL Scheme. In some specific cases we instead choose to use terms
+that occur in the Things SQL database to closer reflect its underlying
+data structures. Whenever that happens, we define the new term here.
+"""
 
 import os
 from shlex import quote
 import sys
-from typing import List, Dict
+from typing import Dict, List
 
-from .database import Database
+from things.database import Database
 
 
 # --------------------------------------------------
@@ -35,62 +38,74 @@ def tasks(uuid=None, include_items=False, **kwargs):  # noqa: C901
     uuid : str or None, optional
         Any valid task uuid. If None, then return all tasks matched.
 
-    include_items : boolean, default False
+    include_items : bool, default False
         Include items contained within a task. These might include
         checklist items, headings, and to-dos.
 
     type : {'to-do', 'heading', 'project', None}, optional
         Only return a specific type of task:
-
-        'to-do':    may have a checklist; may be in an area and have tags.
-        'project':  may have to-dos and headings; may be in an area and
-                    have tags.
-        'heading':  part of a project; groups tasks.
-         None:      return all types of tasks.
+        - `'to-do'`:    may have a checklist; may be in an area and have tags.
+        - `'project'`:  may have to-dos and headings; may be in an area and
+                        have tags.
+        - `'heading'`:  part of a project; groups tasks.
+        - `None` (default): return all types of tasks.
 
     status : {'incomplete', 'completed', 'canceled', None}, optional, \
         default 'incomplete'
 
-        Only include tasks matching that status. If the argument is `None`,
+        Only include tasks matching that status. If `status == None`,
         then include tasks with any status value.
 
     start : {'Inbox', 'Anytime', 'Someday', None}, optional
         Only include tasks matching that start value. If the argument is
         `None` (default), then include tasks with any start value.
 
-    area : str, bool, or None, optional
+    area : str or bool or None, optional
         Any valid uuid of an area. Only include tasks matching that area.
-        If the argument is `False`, only include tasks _without_ an area.
-        If the argument is `True`, only include tasks _with_ an area.
-        If the argument is `None` (default), then include all tasks.
+        Special cases:
+        - `area == False`, only include tasks _without_ an area.
+        - `area == True`, only include tasks _with_ an area.
+        - `area == None` (default), then include all tasks.
 
-    project : str or None, optional
+    project : str or bool or None, optional
         Any valid uuid of a project. Only include tasks matching that project.
-        If the argument is `False`, only include tasks _without_ a project.
-        If the argument is `True`, only include tasks _with_ a project.
-        If the argument is `None` (default), then include all tasks.
+        Special cases:
+        - `project == False`, only include tasks _without_ a project.
+        - `project == True`, only include tasks _with_ a project.
+        - `project == None` (default), then include all tasks.
 
     heading : str or None, optional
         Any valid uuid of a heading. Only include tasks matching that heading.
-        If the argument is `False`, only include tasks _without_ a heading.
-        If the argument is `True`, only include tasks _with_ a heading.
-        If the argument is `None` (default), then include all tasks.
+        Special cases:
+        - `heading == False`, only include tasks _without_ a heading.
+        - `heading == True`, only include tasks _with_ a heading.
+        - `heading == None` (default), then include all tasks.
 
-    tag : str or None, optional
+    tag : str or bool or None, optional
         Any valid title of a tag. Only include tasks matching that tag.
-        If the argument is `False`, only include tasks _without_ tags.
-        If the argument is `True`, only include tasks _with_ tags.
-        If the argument is `None` (default), then include all tasks.
+        Special cases:
+        - `tag == False`, only include tasks _without_ tags.
+        - `tag == True`, only include tasks _with_ tags.
+        - `tag == None` (default), then include all tasks.
 
     start_date : bool or None, optional
-        If the argument is `False`, only include tasks _without_ a start date.
-        If the argument is `True`, only include tasks _with_ a start date.
-        If the argument is `None` (default), then include all tasks.
+        - `start_date == False`, only include tasks _without_ a start date.
+        - `start_date == True`, only include tasks _with_ a start date.
+        - `start_date == None` (default), then include all tasks.
 
     deadline : bool or None, optional
-        If the argument is `False`, only include tasks _without_ a deadline.
-        If the argument is `True`, only include tasks _with_ a deadline.
-        If the argument is `None` (default), then include all tasks.
+        - `deadline == False`, only include tasks _without_ a deadline.
+        - `deadline == True`, only include tasks _with_ a deadline.
+        - `deadline == None` (default), then include all tasks.
+
+    last : str, optional
+        Limit returned tasks to tasks created in the last X days or weeks.
+
+        Takes as input an offset string of format 'X[d/w]' where X is a
+        non-negative  integer that is followed by 'd' or 'w' which stand
+        for 'days' and 'weeks', respectively.
+
+        For example: '3d' or '5w'.
 
     search_query : str, optional
         The string value is passed to the SQL LIKE operator. It can thus
@@ -100,24 +115,19 @@ def tasks(uuid=None, include_items=False, **kwargs):  # noqa: C901
         Currently titles and notes of to-dos, projects, headings, and areas
         are taken into account.
 
-        If the query is `False`, then return items where titles or notes
-        are empty.
-
-        If the query `None`, then return all items.
-
     index : {'index', 'todayIndex'}, default 'index'
         Database field to order result by.
 
-    count_only : boolean, default False
+    count_only : bool, default False
         Only output length of result. This is done by a SQL COUNT query.
 
     filepath : str, optional
         Any valid path of a SQLite database file generated by the Things app.
-        If the environment variable THINGSDB is set, then that path will be used.
+        If the environment variable `THINGSDB` is set, then use that path.
         Otherwise, access the default database path.
 
     database : things.database.Database, optional
-        Any valid `things.database.Database` object previously instantiated.
+        Any valid database object previously instantiated.
 
     Returns
     -------
@@ -125,7 +135,7 @@ def tasks(uuid=None, include_items=False, **kwargs):  # noqa: C901
         Representing multiple tasks.
     dict (if `uuid` is given)
         Representing a single task.
-    int (count_only == True)
+    int (`count_only == True`)
         Count of matching Tasks.
 
     Examples
@@ -138,6 +148,8 @@ def tasks(uuid=None, include_items=False, **kwargs):  # noqa: C901
     ...
     >>> things.tasks(status='completed', count_only=True)
     10
+    >>> things.tasks(status='completed', last='1w' count_only=True)
+    2
 
     """
     database = pop_database(kwargs)
@@ -200,17 +212,18 @@ def areas(uuid=None, include_items=False, **kwargs):
     uuid : str or None, optional
         Any valid uuid of an area. If None, then return all areas.
 
-    include_items : boolean, default False
+    include_items : bool, default False
         Include tasks and projects in each area.
 
-    tag : str or None, optional
+    tag : str or bool or None, optional
         Any valid title of a tag. Only include areas matching that tag.
-        If the argument is `False`, only include areas _without_ tags.
-        If the argument is `True`, only include areas _with_ tags.
-        If the argument is `None`, then ignore any tags present, that is,
-        include areas both with and without tags.
+        Special cases:
+        - `tag == False`, only include areas _without_ tags.
+        - `tag == True`, only include areas _with_ tags.
+        - `tag == None`, then ignore any tags present, that is,
+           include areas both with and without tags.
 
-    count_only : boolean, default False
+    count_only : bool, default False
         Only output length of result. This is done by a SQL COUNT query.
 
     filepath : str, optional
@@ -226,7 +239,7 @@ def areas(uuid=None, include_items=False, **kwargs):
         Representing Things areas.
     dict (if `uuid` is given)
         Representing a single Things area.
-    int (count_only == True)
+    int (`count_only == True`)
         Count of matching areas.
 
     Examples
@@ -270,7 +283,7 @@ def tags(title=None, include_items=False, **kwargs):
         Any valid title of a tag. Include all items of said tag.
         If None, then return all tags.
 
-    include_items : boolean, default False
+    include_items : bool, default False
         For each tag, include items tagged with that tag.
         Items may include areas, tasks, and projects.
 
@@ -288,7 +301,7 @@ def tags(title=None, include_items=False, **kwargs):
         If no path is provided, then access the default database path.
 
     database : things.database.Database, optional
-        Any valid `things.database.Database` object previously instantiated.
+        Any valid database object previously instantiated.
 
     Returns
     -------
@@ -392,7 +405,7 @@ def todos(uuid=None, **kwargs):
     """
     Read to-dos into dicts.
 
-    See `things.api.tasks` for details.
+    See `things.api.tasks` for details on the optional parameters.
     """
     return tasks(uuid=uuid, type="to-do", **kwargs)
 
@@ -401,22 +414,29 @@ def projects(uuid=None, **kwargs):
     """
     Read projects into dicts.
 
-    See `things.api.tasks` for details.
+    See `things.api.tasks` for details on the optional parameters.
     """
     return tasks(uuid=uuid, type="project", **kwargs)
 
 
 def checklist_items(uuid, **kwargs):
     """
-    Read checklist items into dicts.
-    See `things.api.tasks` for details.
+    Read checklist items of a to-do into dicts.
+
+    Note: this checklist is also contained in the return value of
+    `things.todos(uuid)` and `things.tasks(uuid)`.
+
+    Parameters
+    ----------
+    uuid : str
+        A valid to-do uuid.
 
     Returns
     -------
     list of dict
-        Representing checklist items. Or [] if none found.
+        Checklist items of the to-do. If none, then return empty list.
     """
-    return tasks(uuid=uuid, type="to-do", **kwargs).get("checklist", [])  # type: ignore
+    return tasks(uuid=uuid, **kwargs).get("checklist", [])  # type: ignore
 
 
 # Filter by collections in the Things app sidebar.
@@ -424,19 +444,23 @@ def checklist_items(uuid, **kwargs):
 
 def inbox(**kwargs):
     """
-    Read inbox into dicts.
+    Read Inbox into dicts.
 
-    See `things.api.tasks` for details.
+    See `things.api.tasks` for details on the optional parameters.
     """
     return tasks(start="Inbox", **kwargs)
 
 
 def today(**kwargs):
     """
+    Read Today's tasks into dicts.
+
     Note: This might not produce desired results if the Things app hasn't
     been opened yet today and the yellow "OK" button clicked for new tasks.
     In general, you can assume that whatever state the Things app was in
     when you last opened it, that's the state reflected here by the API.
+
+    See `things.api.tasks` for details on the optional parameters.
     """
     database = pop_database(kwargs)
     if not database.was_modified_today():
@@ -458,35 +482,39 @@ def today(**kwargs):
 
 def upcoming(**kwargs):
     """
+    Read Upcoming tasks into dicts.
+
     Note: unscheduled tasks with a deadline are not included here.
-    See the `deadline` function instead.
+    See the `things.api.deadline` function instead.
+
+    For details on parameters, see `things.api.tasks`.
     """
     return tasks(start_date=True, start="Someday", **kwargs)
 
 
 def anytime(**kwargs):
     """
-    Read anytime tasks into dicts.
+    Read Anytime tasks into dicts.
 
-    See `things.api.tasks` for details.
+    See `things.api.tasks` for details on the optional parameters.
     """
     return tasks(start="Anytime", **kwargs)
 
 
 def someday(**kwargs):
     """
-    Read someday tasks into dicts.
+    Read Someday tasks into dicts.
 
-    See `things.api.tasks` for details.
+    See `things.api.tasks` for details on the optional parameters.
     """
     return tasks(start_date=False, start="Someday", **kwargs)
 
 
 def logbook(**kwargs):
     """
-    Read logbook tasks into dicts.
+    Read Logbook tasks into dicts.
 
-    See `things.api.tasks` for details.
+    See `things.api.tasks` for details on the optional parameters.
     """
     result = [*canceled(**kwargs), *completed(**kwargs)]
     result.sort(key=lambda task: task["stop_date"], reverse=True)
@@ -500,19 +528,25 @@ def canceled(**kwargs):
     """
     Read canceled tasks into dicts.
 
-    See `things.api.tasks` for details.
+    See `things.api.tasks` for details on the optional parameters.
     """
     return tasks(status="canceled", **kwargs)
 
 
 def completed(**kwargs):
     """
+    Read completed tasks into dicts.
+
+    See `things.api.tasks` for details on the optional parameters.
+
     Examples
     --------
     >>> things.completed(count_only=True)
     14
     >>> things.completed(type='project', count_only=True)
     4
+    >>> things.completed(type='to-do', last='1w')
+    ...
     """
     return tasks(status="completed", **kwargs)
 
@@ -521,16 +555,65 @@ def deadlines(**kwargs):
     """
     Read tasks with deadlines into dicts.
 
-    See `things.api.tasks` for details.
+    See `things.api.tasks` for details on the optional parameters.
     """
     result = tasks(deadline=True, **kwargs)
     result.sort(key=lambda task: task["deadline"])
     return result
 
 
+def last(offset, **kwargs):
+    """
+    Read tasks created within last X days, weeks, or years into dicts.
+
+    Per default, only incomplete tasks are included, but do see
+    `things.api.tasks` for details on the optional parameters.
+
+    Parameters
+    ----------
+    offset : str
+        A valid date offset such as '3d', '5w', or '1y'.
+        For details, see the `last` parameter of `things.api.tasks`.
+
+    Returns
+    -------
+    list of dict
+        Tasks within offset ordered by creation date. Newest first.
+
+    Examples
+    --------
+    >>> things.last('3d')
+    ...
+    >>> things.last('1w', status='completed')
+    ...
+    >>> things.last('1y', tag='Important', status='completed', count_only=True)
+    8
+
+    """
+    if offset is None:
+        raise ValueError(f"Invalid offset type: {offset!r}")
+
+    result = tasks(last=offset, **kwargs)
+    result.sort(key=lambda task: task["created"], reverse=True)
+
+    return result
+
+
 # Interact with Things app
 
 
+def token(**kwargs) -> str:
+    """
+    Read the Things URL scheme authentication token.
+
+    You can make good use of this token to modify existing Things data
+    using the Things URL Scheme. For details, see
+    [here](https://culturedcode.com/things/help/url-scheme/).
+    """
+    database = pop_database(kwargs)
+    return database.get_url_scheme_auth_token()
+
+
 def link(uuid):
     """
     Returns a things:///show?id=uuid link.
